var proauth = (function () {
'use strict';

class SessionContainer {

  /**
   * Create a session container.
   *
   * @param {String} namespace The namespace
   * @param {Storage} storage The session storage
   */
  constructor(namespace, storage) {
    this.namespace = namespace;
    this.storage = storage;
  }

  get content() {
    try {
      return JSON.parse(this.storage.getItem(this.namespace))
    } catch (error) {
    }
    return null
  }

  set content(data) {
    if (data) {
      this.storage.setItem(this.namespace, JSON.stringify(data || null));
    } else {
      this.clear();
    }
  }

  clear() {
    this.storage.removeItem(this.namespace);
  }

}

class OAuth2Response {

  /**
   * Create an OAuth2.0 response.
   *
   * @param {Response} data The response
   */
  constructor(data) {
    this.accessToken = data.access_token;
    this.refreshToken = data.refresh_token;
    this.expiresIn = data.expires_in;
    this.tokenType = data.token_type;
    this.scope = data.scope;
  }

  toObject() {
    return Object.assign({}, this)
  }

}

class OAuth2Error {

  /**
   * Create an OAuth2.0 error.
   *
   * @param {Response} response The response
   */
  constructor(response) {
    this.response = response;
  }

  get status() {
    return this.response.status || null
  }

  get title() {
    return this.response.title || null
  }

  get detail() {
    return this.response.detail || null
  }

}

class OAuth2Client {

  /**
   * Create an OAuth2.0 client instance.
   *
   * @param {string} apiUrl The url of OAuth2 endpoint
   * @param {string} clientId A string identifying the instance of the client
   * @param {Function} fetch The fetch function the client will use
   */
  constructor(apiUrl, clientId, fetch) {

    /** @type {string} */
    this.apiUrl = apiUrl;

    /** @type {string} */
    this.clientId = clientId;

    this.fetch = (init) => {
      return fetch(this.apiUrl, init)
    };

  }

  handleAuthenticationResponse(response) {
    if (response.ok) {
      // Authentication OK
      return response.json().then((j) => new OAuth2Response(j))
    }

    // Authentication refused
    return response.json().then((j) => Promise.reject(new OAuth2Error(j)))
  }

  /**
   * Grant an access token.
   *
   * @param {Object} params An object containing the parameters to post to the OAuth2 endpoint
   * @return {Promise} A promise
   * @private
   */
  grantAccessToken(params) {
    return this.fetch({
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      body: JSON.stringify(params)
    })
  }

  /**
   * Grant a token for an user.
   *
   * @param {String} username The user for which to request a token
   * @param {String} password The password to authenticate such user
   * @return {Promise} A promise
   */
  userCredentials(username, password) {
    return this.grantAccessToken({
      "client_id": this.clientId,
      "grant_type": "password",
      username,
      password
    });
  }

  /**
   * Refresh an overdue token.
   *
   * @param {String} refreshToken The token to refresh
   * @return {Promise} A promise
   */
  refreshToken(refreshToken) {
    return this.grantAccessToken({
      "client_id": this.clientId,
      "grant_type": "refresh_token",
      "refresh_token": refreshToken
    });
  }

}

const sendMessage = (serviceWorker, message) => {
  // This wraps the message posting/response in a promise,
  // which will resolve if the response doesn't
  // contain an error, and reject with the error if it does.
  // If you'd prefer, it's possible to call
  // controller.postMessage() and set up the onmessage handler
  // independently of a promise, but this is a convenient wrapper.
  return new Promise((resolve, reject) => {
    const messageChannel = new MessageChannel();
    messageChannel.port1.onmessage = (event) => {
      if (event.data.error) {
        reject(event.data.error);
      } else {
        resolve(event.data);
      }
    };

    // This sends the message data as well as transferring messageChannel.port2
    // to the service worker.
    // The service worker can then use the transferred port to reply via postMessage(),
    // which will in turn trigger the onmessage handler on messageChannel.port1.
    // See https://html.spec.whatwg.org/multipage/workers.html#dom-worker-postmessage.
    serviceWorker.controller.postMessage(message, [messageChannel.port2]);
  })
};

class Client {

  /**
   * @typedef {Object} ClientSettings
   * @property {String} namespace
   * @property {String} oauthUrl
   * @property {String} oauthClientId
   * @property {String} sessionStorage
   */

  /**
   * Create a client.
   *
   * @param {ClientSettings} settings An object containing the client settings
   */
  constructor(settings) {
    const { namespace, sessionStorage } = settings;
    this.settings = settings;
    this.sessionContainer = new SessionContainer(namespace, window[sessionStorage]);
    this._onMessage = (event) => {
      const { broadcast, namespace: ns, command, params } = event.data;
      if (!broadcast || ns !== this.settings.namespace) {
        return
      }

      switch (command) {
        case "session":
          this.sessionContainer.content = params[0];
          break;

        default:
      }

      // (todo): set session race condition must be avoided, dirty-checking data may be a solution
    };
  }

  get ready() {
    return this._serviceWorker && this._ready
  }

  get legacyMode() {
    return this._legacyMode
  }

  set legacyMode(sw) {
    throw Error("legacyMode is a read-only property")
  }

  get serviceWorker() {
    if (!this._serviceWorker) {
      throw Error("proauth.js: serviceWorker has been not registered yet.")
    }
    return this._serviceWorker;
  }

  set serviceWorker(sw) {
    if (this._serviceWorker) {
      this._serviceWorker.removeEventListener("message", this._onMessage);
    }

    this._ready = false;
    this._serviceWorker = sw;
    this._legacyMode = sw !== navigator.serviceWorker;

    sendMessage(sw, {
      namespace: this.settings.namespace,
      command: "init",
      params: [this.settings, this.sessionContainer.content]
    }).then((data) => {
      this._serviceWorker.addEventListener("message", this._onMessage);
      this._ready = !!data;
      // (todo): dispatch global ready event
    });
  }

  setSession(data) {
    return sendMessage(this.serviceWorker, {
      namespace: this.settings.namespace,
      command: "setSession",
      params: [data]
    })
  }

  clearSession() {
    return this.setSession(null)
  }

  hasSession() {
    return !!this.sessionContainer.content;
  }

  login(user, pass) {
    if (!this.ready) {
      throw Error("proauth.js is not ready yet.")
    }
    const { oauthUrl, oauthClientId } = this.settings;
    return (new OAuth2Client(oauthUrl, oauthClientId, window.fetch))
      .userCredentials(user, pass)
      .then((response) => response.ok ? response : Promise.reject(response))
  }

}

var absolutePath = (href) => {
  const link = document.createElement("a");
  link.href = href;
  return `${link.protocol}//${link.host}${link.pathname}${link.search}${link.hash}`
};

const boot = () => {
  const settings = {
    // Put defaults here
    swSrc: "/service-worker.js",
    swOptions: {},
    oauthUrl: "/oauth",
    oauthClientId: "proauth",
    sessionStorage: "localStorage",
    namespace: document.origin && document.origin !== "null" ? document.origin : "",
    managedUrls: [
      "/" // Manage the current url root by default
    ]
  };

  // Copy user settings
  Object.assign(settings, window.proauth || {});

  // Patch settings
  settings.oauthUrl = absolutePath(settings.oauthUrl);
  for (const k in settings.managedUrls) {
    settings.managedUrls[k] = absolutePath(settings.managedUrls[k]);
  }

  const client = new Client(settings);

  return client;
};

var proauthClient = {
  client: boot()
};

const client = proauthClient.client;

if (navigator.serviceWorker) {

  if (client.settings.swSrc) {
    navigator.serviceWorker.register(client.settings.swSrc, client.settings.swOptions);
  }

  navigator.serviceWorker.ready.then((registration) => {
    // SW is ready, but its state can be either activating or activated.
    // We must wait until it's activated, so the clients will be claimed.

    if (registration.active.state === "activated") {

      client.serviceWorker = navigator.serviceWorker;

    } else {

      registration.active.addEventListener("statechange", () => {
        if (registration.active.state === "activated") {

          client.serviceWorker = navigator.serviceWorker;

        }
      });

    }
  });
}

var _default = { client };

return _default;

}());
//# sourceMappingURL=default.es2015.js.map
