(function (exports) {
'use strict';

class OAuth2Response {

  /**
   * Create an OAuth2.0 response.
   *
   * @param {Response} data The response
   */
  constructor(data) {
    this.accessToken = data.access_token;
    this.refreshToken = data.refresh_token;
    this.expiresIn = data.expires_in;
    this.tokenType = data.token_type;
    this.scope = data.scope;
  }

  toObject() {
    return Object.assign({}, this)
  }

}

class OAuth2Error {

  /**
   * Create an OAuth2.0 error.
   *
   * @param {Response} response The response
   */
  constructor(response) {
    this.response = response;
  }

  get status() {
    return this.response.status || null
  }

  get title() {
    return this.response.title || null
  }

  get detail() {
    return this.response.detail || null
  }

}

class OAuth2Client {

  /**
   * Create an OAuth2.0 client instance.
   *
   * @param {string} apiUrl The url of OAuth2 endpoint
   * @param {string} clientId A string identifying the instance of the client
   * @param {Function} fetch The fetch function the client will use
   */
  constructor(apiUrl, clientId, fetch) {

    /** @type {string} */
    this.apiUrl = apiUrl;

    /** @type {string} */
    this.clientId = clientId;

    this.fetch = (init) => {
      return fetch(this.apiUrl, init)
    };

  }

  handleAuthenticationResponse(response) {
    if (response.ok) {
      // Authentication OK
      return response.json().then((j) => new OAuth2Response(j))
    }

    // Authentication refused
    return response.json().then((j) => Promise.reject(new OAuth2Error(j)))
  }

  /**
   * Grant an access token.
   *
   * @param {Object} params An object containing the parameters to post to the OAuth2 endpoint
   * @return {Promise} A promise
   * @private
   */
  grantAccessToken(params) {
    return this.fetch({
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      body: JSON.stringify(params)
    })
  }

  /**
   * Grant a token for an user.
   *
   * @param {String} username The user for which to request a token
   * @param {String} password The password to authenticate such user
   * @return {Promise} A promise
   */
  userCredentials(username, password) {
    return this.grantAccessToken({
      "client_id": this.clientId,
      "grant_type": "password",
      username,
      password
    });
  }

  /**
   * Refresh an overdue token.
   *
   * @param {String} refreshToken The token to refresh
   * @return {Promise} A promise
   */
  refreshToken(refreshToken) {
    return this.grantAccessToken({
      "client_id": this.clientId,
      "grant_type": "refresh_token",
      "refresh_token": refreshToken
    });
  }

}

const attachAuthorization = (session, request) => {
  const tokens = session.content || {};

  if (tokens.accessToken) {

    /*
     * We have to create a new request based on the old one because the headers of the
     * original request are immutable in ServiceWorker.
     * Note that this is unnecessary if the code doesn't run in the ServiceWorker.
     * TODO: search for a better way to achieve the same result.
     */
    request = new Request(request);

    request.headers.set("Authorization", `${tokens.tokenType} ${tokens.accessToken}`);
  }

  return request
};

const updateSession = (session, grantPromise) => {
  return grantPromise.then((oA2Response) => {
    session.content = oA2Response.toObject();
    return true
  }, (oA2Error) => {
    session.clear();
    return false
  })
};

class Oauth2Handler {

  /**
   * Create an OAuth2.0 handler.
   *
   * @param {Object} settings The settings
   * @param {SessionHandler} session The session
   * @param {Function} fetch The fetch function
   */
  constructor(settings, session, fetch) {
    this.settings = settings;
    this.session = session;
    this.fetch = fetch;
    this.oauth2Client = new OAuth2Client(settings.oauthUrl, settings.oauthClientId, fetch);
    this.invalidGrantHttpCode = 401;
  }

  /**
   * Update the session on response
   *
   * @param {FetchEvent} event The intercepted event
   * @returns {void}
   */
  handleAuthentication(event) {
    const fetch = this.fetch;
    event.respondWith(fetch(event.request).then((response) => {
      // Just update the session on response
      return updateSession(
        this.session,
        this.oauth2Client.handleAuthenticationResponse(response.clone())
      ).then(() => response)
    }));
  }

  /**
   * Set the current token and handle 401s
   *
   * @param {FetchEvent} event The intercepted event
   * @returns {void}
   */
  handle(event) {

    const { oauthUrl } = this.settings;
    let request = event.request;
    const url = request.url;
    const fetch = this.fetch;

    // Is it an auth request?
    if (url === oauthUrl || `${url}/` === oauthUrl) {
      this.handleAuthentication(event);
      return
    }

    if (this._pendingRefresh) {
      // Wait for the fresh token
      this._pendingRefresh.then(() => {
        request = attachAuthorization(this.session, request);
        event.respondWith(fetch(request));
      });
      return
    }

    // Attach the current token
    request = attachAuthorization(this.session, request);

    // Finally, fetch and handle the response
    let requestCopy = request.clone();
    event.respondWith(fetch(request).then((response) => {

      if (this._pendingRefresh) {
        // Wait for the fresh token
        return this._pendingRefresh.then(() => {
          requestCopy = attachAuthorization(this.session, requestCopy);
          return fetch(requestCopy)
        })
      }

      const tokens = this.session.content || {};
      if (response.status === this.invalidGrantHttpCode && tokens.refreshToken) {
        // Got 401, but we have a refresh token, so try to get a new access token

        const resetPending = (param) => {
          this._pendingRefresh = null;
          return param
        };

        const oauth = this.oauth2Client;
        this._pendingRefresh = oauth.refreshToken(tokens.refreshToken).then(
          resetPending,
          resetPending
        ).then((authResponse) => {
          return updateSession(
            this.session,
            oauth.handleAuthenticationResponse(authResponse)
          ).then((newSession) => {
            // We got a new token?
            // Then try to re-fetch the original request...
            if (newSession) {
              requestCopy = attachAuthorization(this.session, requestCopy);
              return fetch(requestCopy)
            } //...else passthrough the 401 response
            return response
          })
        });
        return this._pendingRefresh
      }
      return response // Passthrough any others
    }));
  }

}

class SessionContainer {

  /**
   * Create a session container.
   *
   * @param {String} namespace The namespace
   * @param {Function} notify The notifier
   * @param {Object} data The content
   */
  constructor(namespace, notify, data) {
    this.notify = () => notify(namespace);
    this._content = data ? Object(data) : null;
  }

  get content() {
    return this._content
  }

  set content(data) {
    this._content = data ? Object(data) : null;
    this.notify();
  }

  clear() {
    this.content = null;
  }

}

class Controller {

  /**
   * Create a controller.
   *
   * @param {EventTarget} globalScope The Service Worker's global scope
   * @param {Function} fetch The fetch function it will use
   */
  constructor(globalScope, fetch) {

    this.globalScope = globalScope;

    if (!fetch) {
      fetch = globalScope.fetch;
    }

    this.handlers = {};

    // Setup interceptor
    globalScope.addEventListener("fetch", (event) => {
      for (const ns in this.handlers) {
        const handler = this.handlers[ns];
        for (const url of handler.settings.managedUrls) {
          if (event.request.url.startsWith(url)) {
            // Each managed url should be handle by just one handler,
            // further registered handler on the same url are discarded.
            // (todo): Should be a warning/error thrown in case of multiple handlers?
            handler.handle(event);
            return
          }
        }
      }

      // Passthrough unmanaged requests
      event.respondWith(fetch(event.request));
    });


    globalScope.addEventListener("message", (e) => {
      const { broadcast, namespace, command, params } = e.data;
      const reply = (...args) => e.ports[0].postMessage(...args);

      if (broadcast) {
        return // Discard broadcast messages
      }

      try {
        switch (command) {
        case "init":
          ((settings, sessionData = {}) => {
            const ns = settings.namespace;
            let h = this.handlers[ns];

            if (namespace !== ns) {
              throw Error("Invalid settings: namespace mismatch")
            }

            // Setup or override handler settings and session data
            if (h) {
              h = this.getHandlerByNamespace(ns);
              h.settings = settings;
              h.session.content = sessionData;
            } else {
              const s = new SessionContainer(ns, this.notifySession.bind(this), sessionData);
              h = this.handlers[ns] = new Oauth2Handler(settings, s, fetch);
            }

          })(...params);
          reply(true);
          break;

        case "setSession":
          ((data) => {
            this.getHandlerByNamespace(namespace).session.content = data;
          })(...params);
          reply(true);
          break;

        default:
          break;
        }
      } catch (error) {
        reply({
          namespace,
          error: {
            message: String(error.message),
            stack: String(error.stack)
          }
        });
      }
    });
  }

  getHandlerByNamespace(namespace) {
    if (!this.handlers[namespace]) {
      throw Error(`No handler found for '${namespace}'`)
    }
    return this.handlers[namespace]
  }

  sendMessage(namespace, command, ...params) {
    this.globalScope.clients.matchAll().then((clients) => {
      for (const client of clients) {
        client.postMessage({
          broadcast: true, namespace, command, params
        });
      }
    });
  }

  notifySession(namespace) {
    this.sendMessage(namespace, "session", this.getHandlerByNamespace(namespace).session.content);
  }
}

self.addEventListener("install", (event) => {
  event.waitUntil(self.skipWaiting());
});

self.addEventListener("activate", (event) => {
  event.waitUntil(self.clients.claim());
});

const controller = new Controller(self, fetch);

const service = { controller };

exports.service = service;

}((this.proauth = this.proauth || {})));
//# sourceMappingURL=service-worker.es2015.js.map
